@@ -24,39 +24,42 @@ import errno
 import json
 import logging
 import operator
-import os
 import re
 import subprocess
 import sys
-from pathlib import Path
 from typing import TYPE_CHECKING
 
 from sqlalchemy import func, select
 
+from airflow._shared.timezones import timezone
 from airflow.api.client import get_current_api_client
 from airflow.api_fastapi.core_api.datamodels.dags import DAGResponse
 from airflow.cli.simple_table import AirflowConsole
 from airflow.cli.utils import fetch_dag_run_from_run_id_or_logical_date_string
 from airflow.dag_processing.bundles.manager import DagBundlesManager
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowConfigException, AirflowException
 from airflow.jobs.job import Job
 from airflow.models import DagBag, DagModel, DagRun, TaskInstance
-from airflow.models.dag import DAG
+from airflow.models.dag import get_next_data_interval
+from airflow.models.dagbag import sync_bag_to_db
 from airflow.models.errors import ParseImportError
 from airflow.models.serialized_dag import SerializedDagModel
-from airflow.sdk.definitions._internal.dag_parsing_context import _airflow_parsing_context_manager
-from airflow.utils import cli as cli_utils, timezone
-from airflow.utils.cli import get_dag, suppress_logs_and_warning, validate_dag_bundle_arg
+from airflow.utils import cli as cli_utils
+from airflow.utils.cli import get_bagged_dag, suppress_logs_and_warning, validate_dag_bundle_arg
 from airflow.utils.dot_renderer import render_dag, render_dag_dependencies
 from airflow.utils.helpers import ask_yesno
+from airflow.utils.platform import getuser
 from airflow.utils.providers_configuration_loader import providers_configuration_loaded
 from airflow.utils.session import NEW_SESSION, create_session, provide_session
 from airflow.utils.state import DagRunState
 
 if TYPE_CHECKING:
+    from collections.abc import Iterable
+
     from graphviz.dot import Dot
     from sqlalchemy.orm import Session
 
+    from airflow import DAG
     from airflow.timetables.base import DataInterval
 
 DAG_DETAIL_FIELDS = {*DAGResponse.model_fields, *DAGResponse.model_computed_fields}
@@ -69,12 +72,18 @@ log = logging.getLogger(__name__)
 def dag_trigger(args) -> None:
     """Create a dag run for the specified dag."""
     api_client = get_current_api_client()
+    try:
+        user = getuser()
+    except AirflowConfigException as e:
+        log.warning("Failed to get user name from os: %s, not setting the triggering user", e)
+        user = None
     try:
         message = api_client.trigger_dag(
             dag_id=args.dag_id,
             run_id=args.run_id,
             conf=args.conf,
             logical_date=args.logical_date,
+            triggering_user_name=user,
             replace_microseconds=args.replace_microseconds,
         )
         AirflowConsole().print_as(
@@ -119,21 +128,18 @@ def dag_unpause(args) -> None:
 
 
 @providers_configuration_loaded
-def set_is_paused(is_paused: bool, args) -> None:
+@provide_session
+def set_is_paused(is_paused: bool, args, *, session: Session = NEW_SESSION) -> None:
     """Set is_paused for DAG by a given dag_id."""
-    should_apply = True
-    with create_session() as session:
-        query = select(DagModel)
+    query = select(DagModel)
+    if args.treat_dag_id_as_regex:
+        query = query.where(DagModel.dag_id.regexp_match(args.dag_id))
+    else:
+        query = query.where(DagModel.dag_id == args.dag_id)
 
-        if args.treat_dag_id_as_regex:
-            query = query.where(DagModel.dag_id.regexp_match(args.dag_id))
-        else:
-            query = query.where(DagModel.dag_id == args.dag_id)
-
-        query = query.where(DagModel.is_paused != is_paused)
-
-        matched_dags = session.scalars(query).all()
+    query = query.where(DagModel.is_paused != is_paused)
 
+    matched_dags: list[DagModel] = session.scalars(query).all()
     if not matched_dags:
         print(f"No {'un' if is_paused else ''}paused DAGs were found")
         return
@@ -145,18 +151,20 @@ def set_is_paused(is_paused: bool, args) -> None:
             f"{','.join(dags_ids)}"
             f"\n\nAre you sure? [y/n]"
         )
-        should_apply = ask_yesno(question)
+        if not ask_yesno(question):
+            print("Operation cancelled by user")
+            return
 
-    if should_apply:
-        for dag_model in matched_dags:
-            dag_model.set_is_paused(is_paused=is_paused)
+    def _update_is_paused(dag_model: DagModel) -> bool:
+        old_is_paused = dag_model.is_paused
+        dag_model.is_paused = is_paused
+        return old_is_paused
 
-        AirflowConsole().print_as(
-            data=[{"dag_id": dag.dag_id, "is_paused": not dag.get_is_paused()} for dag in matched_dags],
-            output=args.output,
-        )
-    else:
-        print("Operation cancelled by user")
+    old_values = [
+        {"dag_id": dag_model.dag_id, "is_paused": _update_is_paused(dag_model)} for dag_model in matched_dags
+    ]
+    session.commit()
+    AirflowConsole().print_as(data=old_values, output=args.output)
 
 
 @providers_configuration_loaded
@@ -186,7 +194,11 @@ def dag_dependencies_show(args) -> None:
 @providers_configuration_loaded
 def dag_show(args) -> None:
     """Display DAG or saves its graphic representation to the file."""
-    dag = get_dag(bundle_names=None, dag_id=args.dag_id, from_db=True)
+    from airflow.models.serialized_dag import SerializedDagModel
+
+    if not (dag := SerializedDagModel.get_dag(dag_id=args.dag_id)):
+        raise SystemExit(f"Can not find dag {args.dag_id!r} in database")
+
     dot = render_dag(dag)
     filename = args.save
     imgcat = args.imgcat
@@ -225,16 +237,18 @@ def _save_dot_to_file(dot: Dot, filename: str) -> None:
     print(f"File {filename} saved")
 
 
-def _get_dagbag_dag_details(dag: DAG) -> dict:
+def _get_dagbag_dag_details(dag: DAG, session: Session) -> dict:
     """Return a dagbag dag details dict."""
+    dag_model: DagModel | None = session.get(DagModel, dag.dag_id)
     return {
         "dag_id": dag.dag_id,
         "dag_display_name": dag.dag_display_name,
-        "bundle_name": dag.get_bundle_name(),
-        "bundle_version": dag.get_bundle_version(),
-        "is_paused": dag.get_is_paused(),
-        "is_stale": dag.get_is_stale(),
+        "bundle_name": dag_model.bundle_name if dag_model else None,
+        "bundle_version": dag_model.bundle_version if dag_model else None,
+        "is_paused": dag_model.is_paused if dag_model else None,
+        "is_stale": dag_model.is_stale if dag_model else None,
         "last_parsed_time": None,
+        "last_parse_duration": None,
         "last_expired": None,
         "relative_fileloc": dag.relative_fileloc,
         "fileloc": dag.fileloc,
@@ -296,14 +310,18 @@ def dag_next_execution(args) -> None:
     >>> airflow dags next-execution tutorial
     2018-08-31 10:38:00
     """
-    dag = get_dag(bundle_names=None, dag_id=args.dag_id, from_db=True)
+    from airflow.models.serialized_dag import SerializedDagModel
 
     with create_session() as session:
-        last_parsed_dag: DagModel = session.scalars(
-            select(DagModel).where(DagModel.dag_id == dag.dag_id)
-        ).one()
+        dag = SerializedDagModel.get_dag(args.dag_id, session=session)
+        last_parsed_dag: DagModel | None = session.scalars(
+            select(DagModel).where(DagModel.dag_id == args.dag_id)
+        ).one_or_none()
 
-    if last_parsed_dag.get_is_paused():
+    if not dag or not last_parsed_dag:
+        raise SystemExit(f"DAG: {args.dag_id} does not exist in the database")
+
+    if last_parsed_dag.is_paused:
         print("[INFO] Please be reminded this DAG is PAUSED now.", file=sys.stderr)
 
     def print_execution_interval(interval: DataInterval | None):
@@ -317,7 +335,7 @@ def dag_next_execution(args) -> None:
             return
         print(interval.start.isoformat())
 
-    next_interval = dag.get_next_data_interval(last_parsed_dag)
+    next_interval = get_next_data_interval(dag.timetable, last_parsed_dag)
     print_execution_interval(next_interval)
 
     for _ in range(1, args.num_executions):
@@ -343,15 +361,35 @@ def dag_list_dags(args, session: Session = NEW_SESSION) -> None:
             file=sys.stderr,
         )
 
-    dagbag = DagBag(read_dags_from_db=True)
-    dagbag.collect_dags_from_db()
+    dagbag_import_errors = 0
+    dags_list = []
+    if args.local:
+        from airflow.models.dagbag import DagBag
 
-    # Get import errors from the DB
-    query = select(func.count()).select_from(ParseImportError)
-    if args.bundle_name:
-        query = query.where(ParseImportError.bundle_name.in_(args.bundle_name))
+        # Get import errors from the local area
+        if args.bundle_name:
+            manager = DagBundlesManager()
+            validate_dag_bundle_arg(args.bundle_name)
+            all_bundles = list(manager.get_all_dag_bundles())
+            bundles_to_search = set(args.bundle_name)
 
-    dagbag_import_errors = session.scalar(query)
+            for bundle in all_bundles:
+                if bundle.name in bundles_to_search:
+                    dagbag = DagBag(bundle.path, bundle_path=bundle.path)
+                    dagbag.collect_dags()
+                    dags_list.extend(list(dagbag.dags.values()))
+                    dagbag_import_errors += len(dagbag.import_errors)
+        else:
+            dagbag = DagBag()
+            dagbag.collect_dags()
+            dags_list.extend(list(dagbag.dags.values()))
+            dagbag_import_errors += len(dagbag.import_errors)
+    else:
+        dags_list.extend(sm.dag for sm in session.scalars(select(SerializedDagModel)))
+        pie_stmt = select(func.count()).select_from(ParseImportError)
+        if args.bundle_name:
+            pie_stmt = pie_stmt.where(ParseImportError.bundle_name.in_(args.bundle_name))
+        dagbag_import_errors = session.scalar(pie_stmt)
 
     if dagbag_import_errors > 0:
         from rich import print as rich_print
@@ -365,24 +403,27 @@ def dag_list_dags(args, session: Session = NEW_SESSION) -> None:
     def get_dag_detail(dag: DAG) -> dict:
         dag_model = DagModel.get_dagmodel(dag.dag_id, session=session)
         if dag_model:
-            dag_detail = DAGResponse.from_orm(dag_model).model_dump()
+            dag_detail = DAGResponse.model_validate(dag_model, from_attributes=True).model_dump()
         else:
-            dag_detail = _get_dagbag_dag_details(dag)
+            dag_detail = _get_dagbag_dag_details(dag, session)
         if not cols:
             return dag_detail
         return {col: dag_detail[col] for col in cols if col in DAG_DETAIL_FIELDS}
 
-    def filter_dags_by_bundle(dags: list[DAG], bundle_names: list[str] | None) -> list[DAG]:
+    def filter_dags_by_bundle(dags: Iterable[DAG], bundle_names: list[str] | None) -> Iterable[DAG]:
         """Filter DAGs based on the specified bundle name, if provided."""
         if not bundle_names:
             return dags
 
         validate_dag_bundle_arg(bundle_names)
-        return [dag for dag in dags if dag.get_bundle_name() in bundle_names]
+        selected_dag_ids = set(
+            session.scalars(select(DagModel.dag_id).where(DagModel.bundle_name.in_(bundle_names)))
+        )
+        return (dag for dag in dags if dag.dag_id in selected_dag_ids)
 
     AirflowConsole().print_as(
         data=sorted(
-            filter_dags_by_bundle(list(dagbag.dags.values()), args.bundle_name),
+            filter_dags_by_bundle(dags_list, args.bundle_name if not args.local else None),
             key=operator.attrgetter("dag_id"),
         ),
         output=args.output,
@@ -420,22 +461,41 @@ def dag_list_import_errors(args, session: Session = NEW_SESSION) -> None:
     """Display dags with import errors on the command line."""
     data = []
 
-    # Get import errors from the DB
-    query = select(ParseImportError)
-    if args.bundle_name:
-        validate_dag_bundle_arg(args.bundle_name)
-        query = query.where(ParseImportError.bundle_name.in_(args.bundle_name))
+    if args.local:
+        # Get import errors from local areas
+        if args.bundle_name:
+            manager = DagBundlesManager()
+            validate_dag_bundle_arg(args.bundle_name)
+            all_bundles = list(manager.get_all_dag_bundles())
+            bundles_to_search = set(args.bundle_name)
 
-    dagbag_import_errors = session.scalars(query).all()
+            for bundle in all_bundles:
+                if bundle.name in bundles_to_search:
+                    dagbag = DagBag(bundle.path, bundle_path=bundle.path)
+                    for filename, errors in dagbag.import_errors.items():
+                        data.append({"bundle_name": bundle.name, "filepath": filename, "error": errors})
+        else:
+            dagbag = DagBag()
+            for filename, errors in dagbag.import_errors.items():
+                data.append({"filepath": filename, "error": errors})
 
-    for import_error in dagbag_import_errors:
-        data.append(
-            {
-                "bundle_name": import_error.bundle_name,
-                "filepath": import_error.filename,
-                "error": import_error.stacktrace,
-            }
-        )
+    else:
+        # Get import errors from the DB
+        query = select(ParseImportError)
+        if args.bundle_name:
+            validate_dag_bundle_arg(args.bundle_name)
+            query = query.where(ParseImportError.bundle_name.in_(args.bundle_name))
+
+        dagbag_import_errors = session.scalars(query).all()
+
+        for import_error in dagbag_import_errors:
+            data.append(
+                {
+                    "bundle_name": import_error.bundle_name,
+                    "filepath": import_error.filename,
+                    "error": import_error.stacktrace,
+                }
+            )
     AirflowConsole().print_as(
         data=data,
         output=args.output,
@@ -549,29 +609,6 @@ def dag_list_dag_runs(args, dag: DAG | None = None, session: Session = NEW_SESSI
     AirflowConsole().print_as(data=dag_runs, output=args.output, mapper=_render_dagrun)
 
 
-def _parse_and_get_dag(dag_id: str) -> DAG | None:
-    """Given a dag_id, determine the bundle and relative fileloc from the db, then parse and return the DAG."""
-    db_dag = get_dag(bundle_names=None, dag_id=dag_id, from_db=True)
-    bundle_name = db_dag.get_bundle_name()
-    if bundle_name is None:
-        raise AirflowException(
-            f"Bundle name for DAG {dag_id!r} is not found in the database. This should not happen."
-        )
-    if db_dag.relative_fileloc is None:
-        raise AirflowException(
-            f"Relative fileloc for DAG {dag_id!r} is not found in the database. This should not happen."
-        )
-    bundle = DagBundlesManager().get_bundle(bundle_name)
-    bundle.initialize()
-    dag_absolute_path = os.fspath(Path(bundle.path, db_dag.relative_fileloc))
-
-    with _airflow_parsing_context_manager(dag_id=dag_id):
-        bag = DagBag(
-            dag_folder=dag_absolute_path, include_examples=False, safe_mode=False, load_op_links=False
-        )
-        return bag.dags.get(dag_id)
-
-
 @cli_utils.action_cli
 @providers_configuration_loaded
 @provide_session
@@ -590,19 +627,21 @@ def dag_test(args, dag: DAG | None = None, session: Session = NEW_SESSION) -> No
         re.compile(args.mark_success_pattern) if args.mark_success_pattern is not None else None
     )
 
-    dag = dag or _parse_and_get_dag(args.dag_id)
+    dag = dag or get_bagged_dag(
+        bundle_names=args.bundle_name,
+        dag_id=args.dag_id,
+        dagfile_path=args.dagfile_path,
+    )
     if not dag:
         raise AirflowException(
             f"Dag {args.dag_id!r} could not be found; either it does not exist or it failed to parse."
         )
 
-    dag = DAG.from_sdk_dag(dag)
     dr: DagRun = dag.test(
         logical_date=logical_date,
         run_conf=run_conf,
         use_executor=use_executor,
         mark_success_pattern=mark_success_pattern,
-        session=session,
     )
     show_dagrun = args.show_dagrun
     imgcat = args.imgcat_dagrun
@@ -649,4 +688,4 @@ def dag_reserialize(args, session: Session = NEW_SESSION) -> None:
             continue
         bundle.initialize()
         dag_bag = DagBag(bundle.path, bundle_path=bundle.path, include_examples=False)
-        dag_bag.sync_to_db(bundle.name, bundle_version=bundle.get_current_version(), session=session)
+        sync_bag_to_db(dag_bag, bundle.name, bundle_version=bundle.get_current_version(), session=session)
