@@ -28,7 +28,7 @@ from collections import defaultdict
 from collections.abc import Collection, Iterable
 from datetime import timedelta
 from functools import cache
-from typing import TYPE_CHECKING, Any, cast
+from typing import TYPE_CHECKING, Any
 from urllib.parse import quote
 
 import attrs
@@ -121,7 +121,7 @@ if TYPE_CHECKING:
     from airflow.sdk.definitions.asset import AssetUniqueKey
     from airflow.sdk.types import RuntimeTaskInstanceProtocol
     from airflow.serialization.definitions.taskgroup import SerializedTaskGroup
-    from airflow.serialization.serialized_objects import SerializedBaseOperator, SerializedDAG
+    from airflow.serialization.serialized_objects import SerializedBaseOperator
     from airflow.utils.context import Context
 
     Operator: TypeAlias = MappedOperator | SerializedBaseOperator
@@ -2037,16 +2037,87 @@ class TaskInstance(Base, LoggingMixin):
         *,
         session: Session,
     ) -> int | range | None:
+        """
+        Infer the map indexes of an upstream "relevant" to this ti.
+
+        The bulk of the logic mainly exists to solve the problem described by
+        the following example, where 'val' must resolve to different values,
+        depending on where the reference is being used::
+
+            @task
+            def this_task(v):  # This is self.task.
+                return v * 2
+
+
+            @task_group
+            def tg1(inp):
+                val = upstream(inp)  # This is the upstream task.
+                this_task(val)  # When inp is 1, val here should resolve to 2.
+                return val
+
+
+            # This val is the same object returned by tg1.
+            val = tg1.expand(inp=[1, 2, 3])
+
+
+            @task_group
+            def tg2(inp):
+                another_task(inp, val)  # val here should resolve to [2, 4, 6].
+
+
+            tg2.expand(inp=["a", "b"])
+
+        The surrounding mapped task groups of ``upstream`` and ``self.task`` are
+        inspected to find a common "ancestor". If such an ancestor is found,
+        we need to return specific map indexes to pull a partial value from
+        upstream XCom.
+
+        :param upstream: The referenced upstream task.
+        :param ti_count: The total count of task instance this task was expanded
+            by the scheduler, i.e. ``expanded_ti_count`` in the template context.
+        :return: Specific map index or map indexes to pull, or ``None`` if we
+            want to "whole" return value (i.e. no mapped task groups involved).
+        """
+        from airflow.models.mappedoperator import get_mapped_ti_count
+
         if TYPE_CHECKING:
-            assert self.task
-        return _get_relevant_map_indexes(
-            run_id=self.run_id,
-            map_index=self.map_index,
-            ti_count=ti_count,
-            task=self.task,
-            relative=upstream,
-            session=session,
-        )
+            assert self.task is not None
+
+        # This value should never be None since we already know the current task
+        # is in a mapped task group, and should have been expanded, despite that,
+        # we need to check that it is not None to satisfy Mypy.
+        # But this value can be 0 when we expand an empty list, for that it is
+        # necessary to check that ti_count is not 0 to avoid dividing by 0.
+        if not ti_count:
+            return None
+
+        # Find the innermost common mapped task group between the current task
+        # If the current task and the referenced task does not have a common
+        # mapped task group, the two are in different task mapping contexts
+        # (like another_task above), and we should use the "whole" value.
+        common_ancestor = _find_common_ancestor_mapped_group(self.task, upstream)
+        if common_ancestor is None:
+            return None
+
+        # At this point we know the two tasks share a mapped task group, and we
+        # should use a "partial" value. Let's break down the mapped ti count
+        # between the ancestor and further expansion happened inside it.
+
+        ancestor_ti_count = get_mapped_ti_count(common_ancestor, self.run_id, session=session)
+        ancestor_map_index = self.map_index * ancestor_ti_count // ti_count
+
+        # If the task is NOT further expanded inside the common ancestor, we
+        # only want to reference one single ti. We must walk the actual DAG,
+        # and "ti_count == ancestor_ti_count" does not work, since the further
+        # expansion may be of length 1.
+        if not _is_further_mapped_inside(upstream, common_ancestor):
+            return ancestor_map_index
+
+        # Otherwise we need a partial aggregation for values from selected task
+        # instances in the ancestor's expansion context.
+        further_count = ti_count // ancestor_ti_count
+        map_index_start = ancestor_map_index * further_count
+        return range(map_index_start, map_index_start + further_count)
 
     def clear_db_references(self, session: Session):
         """
@@ -2136,159 +2207,6 @@ def _is_further_mapped_inside(operator: Operator, container: SerializedTaskGroup
     return False
 
 
-def _get_relevant_map_indexes(
-    *,
-    task: Operator,
-    run_id: str,
-    map_index: int,
-    relative: Operator,
-    ti_count: int | None,
-    session: Session,
-) -> int | range | None:
-    """
-    Infer the map indexes of a relative that's "relevant" to this ti.
-
-    The bulk of the logic mainly exists to solve the problem described by
-    the following example, where 'val' must resolve to different values,
-    depending on where the reference is being used::
-
-        @task
-        def this_task(v):  # This is self.task.
-            return v * 2
-
-
-        @task_group
-        def tg1(inp):
-            val = upstream(inp)  # This is the upstream task.
-            this_task(val)  # When inp is 1, val here should resolve to 2.
-            return val
-
-
-        # This val is the same object returned by tg1.
-        val = tg1.expand(inp=[1, 2, 3])
-
-
-        @task_group
-        def tg2(inp):
-            another_task(inp, val)  # val here should resolve to [2, 4, 6].
-
-
-        tg2.expand(inp=["a", "b"])
-
-    The surrounding mapped task groups of ``upstream`` and ``task`` are
-    inspected to find a common "ancestor". If such an ancestor is found,
-    we need to return specific map indexes to pull a partial value from
-    upstream XCom.
-
-    The same logic apply for finding downstream tasks.
-
-    :param task: Current task being inspected.
-    :param run_id: Current run ID.
-    :param map_index: Map index of the current task instance.
-    :param relative: The relative task to find relevant map indexes for.
-    :param ti_count: The total count of task instance this task was expanded
-        by the scheduler, i.e. ``expanded_ti_count`` in the template context.
-    :return: Specific map index or map indexes to pull, or ``None`` if we
-        want to "whole" return value (i.e. no mapped task groups involved).
-    """
-    from airflow.models.mappedoperator import get_mapped_ti_count
-
-    # This value should never be None since we already know the current task
-    # is in a mapped task group, and should have been expanded, despite that,
-    # we need to check that it is not None to satisfy Mypy.
-    # But this value can be 0 when we expand an empty list, for that it is
-    # necessary to check that ti_count is not 0 to avoid dividing by 0.
-    if not ti_count:
-        return None
-
-    # Find the innermost common mapped task group between the current task
-    # If the current task and the referenced task does not have a common
-    # mapped task group, the two are in different task mapping contexts
-    # (like another_task above), and we should use the "whole" value.
-    if (common_ancestor := _find_common_ancestor_mapped_group(task, relative)) is None:
-        return None
-
-    # At this point we know the two tasks share a mapped task group, and we
-    # should use a "partial" value. Let's break down the mapped ti count
-    # between the ancestor and further expansion happened inside it.
-
-    ancestor_ti_count = get_mapped_ti_count(common_ancestor, run_id, session=session)
-    ancestor_map_index = map_index * ancestor_ti_count // ti_count
-
-    # If the task is NOT further expanded inside the common ancestor, we
-    # only want to reference one single ti. We must walk the actual DAG,
-    # and "ti_count == ancestor_ti_count" does not work, since the further
-    # expansion may be of length 1.
-    if not _is_further_mapped_inside(relative, common_ancestor):
-        return ancestor_map_index
-
-    # Otherwise we need a partial aggregation for values from selected task
-    # instances in the ancestor's expansion context.
-    further_count = ti_count // ancestor_ti_count
-    map_index_start = ancestor_map_index * further_count
-    return range(map_index_start, map_index_start + further_count)
-
-
-def find_relevant_relatives(
-    normal_tasks: Iterable[str],
-    mapped_tasks: Iterable[tuple[str, int]],
-    *,
-    direction: Literal["upstream", "downstream"],
-    dag: SerializedDAG,
-    run_id: str,
-    session: Session,
-) -> Collection[str | tuple[str, int]]:
-    from airflow.models.mappedoperator import get_mapped_ti_count
-
-    visited: set[str | tuple[str, int]] = set()
-
-    def _visit_relevant_relatives_for_normal(task_ids: Iterable[str]) -> None:
-        partial_dag = dag.partial_subset(
-            task_ids=task_ids,
-            include_downstream=direction == "downstream",
-            include_upstream=direction == "upstream",
-            exclude_original=True,
-        )
-        visited.update(partial_dag.task_dict)
-
-    def _visit_relevant_relatives_for_mapped(mapped_tasks: Iterable[tuple[str, int]]) -> None:
-        for task_id, map_index in mapped_tasks:
-            task = dag.get_task(task_id)
-            ti_count = get_mapped_ti_count(task, run_id, session=session)
-            # TODO (GH-52141): This should return scheduler operator types, but
-            # currently get_flat_relatives is inherited from SDK DAGNode.
-            relatives = cast("Iterable[Operator]", task.get_flat_relatives(upstream=direction == "upstream"))
-            for relative in relatives:
-                if relative.task_id in visited:
-                    continue
-                relative_map_indexes = _get_relevant_map_indexes(
-                    task=task,
-                    relative=relative,  # type: ignore[arg-type]
-                    run_id=run_id,
-                    map_index=map_index,
-                    ti_count=ti_count,
-                    session=session,
-                )
-                visiting_mapped: set[tuple[str, int]] = set()
-                visiting_normal: set[str] = set()
-                match relative_map_indexes:
-                    case int():
-                        if (item := (relative.task_id, relative_map_indexes)) not in visited:
-                            visiting_mapped.add(item)
-                    case range():
-                        visiting_mapped.update((relative.task_id, i) for i in relative_map_indexes)
-                    case None:
-                        if (task_id := relative.task_id) not in visited:
-                            visiting_normal.add(task_id)
-                _visit_relevant_relatives_for_normal(visiting_normal)
-                _visit_relevant_relatives_for_mapped(visiting_mapped)
-                visited.update(visiting_mapped, visiting_normal)
-
-    _visit_relevant_relatives_for_normal(normal_tasks)
-    _visit_relevant_relatives_for_mapped(mapped_tasks)
-    return visited
-
-
 class TaskInstanceNote(Base):
     """For storage of arbitrary notes concerning the task instance."""
 
