@@ -17,9 +17,11 @@
  * under the License.
  */
 import { FiTag } from "react-icons/fi";
+import { Link as RouterLink } from "react-router-dom";
 
 import type { DagTagResponse } from "openapi/requests/types.gen";
 import { LimitedItemsList } from "src/components/LimitedItemsList";
+import { SearchParamsKeys } from "src/constants/searchParams";
 
 const MAX_TAGS = 3;
 
@@ -31,7 +33,12 @@ type Props = {
 export const DagTags = ({ hideIcon = false, tags }: Props) => (
   <LimitedItemsList
     icon={hideIcon ? undefined : <FiTag data-testid="dag-tag" />}
-    items={tags.map(({ name }) => name)}
+    interactive
+    items={tags.map(({ name }) => (
+      <RouterLink key={name} to={`/dags?${SearchParamsKeys.TAGS}=${encodeURIComponent(name)}`}>
+        {name}
+      </RouterLink>
+    ))}
     maxItems={MAX_TAGS}
   />
 );
