@@ -18,6 +18,9 @@
  */
 import { Box, Heading, Link } from "@chakra-ui/react";
 import type { ColumnDef } from "@tanstack/react-table";
+import type { TFunction } from "i18next";
+import { useMemo } from "react";
+import { useTranslation } from "react-i18next";
 
 import { useProviderServiceGetProviders } from "openapi/queries";
 import type { ProviderResponse } from "openapi/requests/types.gen";
@@ -26,7 +29,7 @@ import { useTableURLState } from "src/components/DataTable/useTableUrlState";
 import { ErrorAlert } from "src/components/ErrorAlert";
 import { urlRegex } from "src/constants/urlRegex";
 
-const columns: Array<ColumnDef<ProviderResponse>> = [
+const createColumns = (translate: TFunction): Array<ColumnDef<ProviderResponse>> => [
   {
     accessorKey: "package_name",
     cell: ({ row: { original } }) => (
@@ -41,13 +44,13 @@ const columns: Array<ColumnDef<ProviderResponse>> = [
       </Link>
     ),
     enableSorting: false,
-    header: "Package Name",
+    header: translate("providers.columns.packageName"),
   },
   {
     accessorKey: "version",
     cell: ({ row: { original } }) => original.version,
     enableSorting: false,
-    header: () => "Version",
+    header: translate("providers.columns.version"),
   },
   {
     accessorKey: "description",
@@ -66,13 +69,16 @@ const columns: Array<ColumnDef<ProviderResponse>> = [
       );
     },
     enableSorting: false,
-    header: "Description",
+    header: translate("columns.description"),
   },
 ];
 
 export const Providers = () => {
+  const { t: translate } = useTranslation(["admin", "common"]);
   const { setTableURLState, tableURLState } = useTableURLState();
 
+  const columns = useMemo(() => createColumns(translate), [translate]);
+
   const { pagination } = tableURLState;
 
   const { data, error } = useProviderServiceGetProviders({
@@ -82,13 +88,13 @@ export const Providers = () => {
 
   return (
     <Box p={2}>
-      <Heading>Providers</Heading>
+      <Heading>{translate("common:admin.Providers")}</Heading>
       <DataTable
         columns={columns}
         data={data?.providers ?? []}
         errorMessage={<ErrorAlert error={error} />}
         initialState={tableURLState}
-        modelName="Provider"
+        modelName={translate("common:admin.Providers")}
         onStateChange={setTableURLState}
         total={data?.total_entries}
       />
